import os

import paramiko
import pywildcard


class SFTPClient:

    def __init__(self, host_config):
        self.host_config = host_config
        self.transport = paramiko.Transport((host_config['hostname'], host_config['port']))
        self.local_path = ''
        self.remote_path = ''
        self.client = None

    def sftp(self):
        self.transport.connect(username=self.host_config['username'],
                               password=self.host_config['password'])
        self.client = paramiko.SFTPClient.from_transport(self.transport)
        return self

    def cd(self, path: str):
        self.remote_path = path
        self.client.chdir(self.remote_path)
        return self

    def lcd(self, path: str):
        self.local_path = path
        return self

    def put(self, file):
        if self.local_path:
            local_file = '{0}/{1}'.format(self.local_path, file)
        self.client.put(local_file, file)
        return self

    def get(self, file):
        if self.local_path:
            local_file = '{0}/{1}'.format(self.local_path, file)
        self.client.get(file, local_file)
        return self

    def mput(self, pattern):
        files = os.listdir(self.local_path)
        matched_files = pywildcard.filter(files, pattern)
        for file in matched_files:
            self.put(file)
        return self

    def mget(self, pattern):
        files = self.client.listdir(self.client.getcwd())
        matched_files = pywildcard.filter(files, pattern)
        for file in matched_files:
            self.get(file)
        return self

    def bye(self):
        self.transport.close()





import paramiko

from util.logger import logger


class SSHClient:

    def __init__(self, host_config:dict):
        self.client = paramiko.SSHClient()
        self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.host_config = host_config

    def ssh(self):
        self.client.connect(hostname=self.host_config['hostname'],
                            port=self.host_config['port'],
                            username=self.host_config['username'],
                            password=self.host_config['password'])
        logger.info('login to {0} -> {1}'.format(self.host_config, 'success'))
        return self

    def shell(self, cmd: str):
        logger.info('[{0}] : {1}'.format(self.host_config['hostname'], cmd))
        stdin, stdout, stderr = self.client.exec_command(cmd)
        res, err = stdout.read(), stderr.read()
        result = res if res else err
        logger.info('[{0}] : {1} -> {2}'.format(self.host_config['hostname'],
                                                cmd,
                                                result.decode()))
        return self

    def exit(self):
        self.client.close()


host_config ={
    'hostname':'122.112.155.31',
    'port':'9022',
    'username':'root',
    'password':'Knownofear!'
}
SSHClient(host_config).ssh().shell('docker info').exit()


from enum import Enum
import yaml
import json

from util.sftp import SFTPClient
from util.ssh import SSHClient


class DeployType(Enum):
    DOCKER = 1
    JAR = 2


class DeployConfig:


    def __init__(self,config):
        self.host_config = config['host_config']


    def to_string(self):
        return json.dump(self)

    def to_yaml(self):
        yaml_path = self.project_name
        with open(yaml_path, 'w', encoding='utf-8') as f:
            yaml.dump(self.to_string())

    @staticmethod
    def of_yaml(yaml_path):
        with open(yaml_path, 'r', encoding='utf-8') as f:
            config = yaml.load(f.read(), Loader=yaml.FullLoader)
            return DeployConfig(config)



class SimpleDeployer():

    def __init__(self, deploy_config: DeployConfig):
        self.host_config = deploy_config.host_config
        self.sftp_client = SFTPClient(host_config=self.host_config)
        self.ssh_client = SSHClient(host_config=self.host_config)

    def upload(self):
        self.upload()

    def deploy(self):
        self.sftp_client.sftp() \
            .cd('') \
            .lcd('') \
            .put() \
            .bye()

        self.ssh_client.ssh() \
            .shell() \
            .shell() \
            .exit()
        return self

    def validate(self):
        return self
